/*
 * bit7z - A C++ static library to interface with the 7-zip shared libraries.
 * Copyright (c) 2014-2022 Riccardo Ostani - All Rights Reserved.
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0. If a copy of the MPL was not distributed with this
 * file, You can obtain one at https://mozilla.org/MPL/2.0/.
 */

#ifndef PROCESSEDITEM_HPP
#define PROCESSEDITEM_HPP

#include "bitinputarchive.hpp"
#include "internal/fs.hpp"
#include "internal/windows.hpp"

namespace bit7z {

class ProcessedItem final {
    public:
        ProcessedItem();

        void loadItemInfo( const BitInputArchive& inputArchive, std::uint32_t itemIndex );

        BIT7Z_NODISCARD auto path() const -> fs::path;

        BIT7Z_NODISCARD auto attributes() const -> uint32_t;

        BIT7Z_NODISCARD auto areAttributesDefined() const -> bool;

        BIT7Z_NODISCARD auto modifiedTime() const -> FILETIME;

        BIT7Z_NODISCARD auto isModifiedTimeDefined() const -> bool;

    private:
        fs::path mFilePath;

        FILETIME mModifiedTime;
        bool mIsModifiedTimeDefined;

        uint32_t mAttributes;
        bool mAreAttributesDefined;

        void loadFilePath( const BitInputArchive& inputArchive, uint32_t itemIndex );

        void loadAttributes( const BitInputArchive& inputArchive, uint32_t itemIndex );

        void loadModifiedTime( const BitInputArchive& inputArchive, uint32_t itemIndex );
};

}  // namespace bit7z

#endif //PROCESSEDITEM_HPP
